@@ -1,20 +1,22 @@
 import numpy as np
 import tensorflow as tf
-from scipy import sparse as sp
 
 from spektral.data.utils import (
     batch_generator,
+    collate_labels_batch,
+    collate_labels_disjoint,
+    get_spec,
     prepend_none,
+    sp_matrices_to_sp_tensors,
     to_batch,
     to_disjoint,
     to_mixed,
     to_tf_signature,
 )
-from spektral.layers.ops import sp_matrix_to_sp_tensor
 
 version = tf.__version__.split(".")
 major, minor = int(version[0]), int(version[1])
-tf_loader_available = major > 1 and minor > 3
+tf_loader_available = major >= 2 and minor >= 4
 
 
 class Loader:
@@ -23,11 +25,11 @@ class Loader:
     Dataset and yield batches of graphs to feed your Keras Models.
 
     This is achieved by having a generator object that produces lists of Graphs,
-    which are then collated together and returned as Tensor-like objects.
+    which are then collated together and returned as Tensors.
 
     The core of a Loader is the `collate(batch)` method.
-    This takes as input a list of Graphs and returns a list of Tensors or
-    SparseTensors.
+    This takes as input a list of `Graph` objects and returns a list of Tensors,
+    np.arrays, or SparseTensors.
 
     For instance, if all graphs have the same number of nodes and size of the
     attributes, a simple collation function can be:
@@ -39,8 +41,8 @@ class Loader:
         return x, a
     ```
 
-    The `load()` method of a Loader returns an object that can be given as
-    input to `Model.fit()`.
+    The `load()` method of a Loader returns an object that can be passed to a Keras
+    model when using the `fit`, `predict` and `evaluate` functions.
     You can use it as follows:
 
     ```python
@@ -48,25 +50,26 @@ class Loader:
     ```
 
     The `steps_per_epoch` property represents the number of batches that are in
-    an epoch, and is a required keyword when calling `model.fit()` with a Loader.
+    an epoch, and is a required keyword when calling `fit`, `predict` or `evaluate`
+    with a Loader.
 
-    If you want to write your own training function, you can use the
-    `tf_signature()` method to specify the signature of your batches using the
-    tf.TypeSpec system, in order to avoid unnecessary re-tracings.
+    If you are using a custom training function, you can specify the input signature
+    of your batches with the tf.TypeSpec system to avoid unnecessary re-tracings.
+    The signature is computed automatically by calling `loader.tf_signature()`.
 
-    For example, a simple training function can be written as:
+    For example, a simple training step can be written as:
 
     ```python
-    @tf.function(input_signature=loader.tf_signature())
+    @tf.function(input_signature=loader.tf_signature())  # Specify signature here
     def train_step(inputs, target):
         with tf.GradientTape() as tape:
             predictions = model(inputs, training=True)
-            loss = loss_fn(target, predictions) + sum(model.losses)
+            loss = loss_fn(target, predictions)
         gradients = tape.gradient(loss, model.trainable_variables)
-        opt.apply_gradients(zip(gradients, model.trainable_variables))
+        optimizer.apply_gradients(zip(gradients, model.trainable_variables))
     ```
 
-    We can then train our model in a for loop as follows:
+    We can then train our model in a loop as follows:
 
     ```python
     for batch in loader:
@@ -75,11 +78,11 @@ class Loader:
 
     **Arguments**
 
-    - `dataset`: a graph Dataset;
-    - `batch_size`: size of the mini-batches;
-    - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
+    - `dataset`: a `spektral.data.Dataset` object;
+    - `batch_size`: int, size of the mini-batches;
+    - `epochs`: int, number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
-    - `shuffle`: whether to shuffle the data at the start of each epoch.
+    - `shuffle`: bool, whether to shuffle the dataset at the start of each epoch.
     """
 
     def __init__(self, dataset, batch_size=1, epochs=None, shuffle=True):
@@ -97,6 +100,9 @@ class Loader:
         return self.collate(nxt)
 
     def generator(self):
+        """
+        Returns lists (batches) of `Graph` objects.
+        """
         return batch_generator(
             self.dataset,
             batch_size=self.batch_size,
@@ -105,65 +111,64 @@ class Loader:
         )
 
     def collate(self, batch):
+        """
+        Converts a list of graph objects to Tensors or np.arrays representing the batch.
+        :param batch: a list of `Graph` objects.
+        """
         raise NotImplementedError
 
     def load(self):
+        """
+        Returns an object that can be passed to a Keras model when using the `fit`,
+        `predict` and `evaluate` functions.
+        By default, returns the Loader itself, which is a generator.
+        """
         return self
 
     def tf_signature(self):
         """
-        Adjacency matrix has shape [n_nodes, n_nodes]
-        Node features have shape [n_nodes, n_node_features]
-        Edge features have shape [n_edges, n_node_features]
-        Targets have shape [..., n_labels]
+        Returns the signature of the collated batches using the tf.TypeSpec system.
+        By default, the signature is that of the dataset (`dataset.signature`):
+
+            - Adjacency matrix has shape `[n_nodes, n_nodes]`
+            - Node features have shape `[n_nodes, n_node_features]`
+            - Edge features have shape `[n_edges, n_node_features]`
+            - Targets have shape `[..., n_labels]`
         """
         signature = self.dataset.signature
         return to_tf_signature(signature)
 
-    def pack(self, batch, return_dict=False):
+    def pack(self, batch):
         """
-        Given a batch of graphs, groups their attributes into separate lists.
+        Given a batch of graphs, groups their attributes into separate lists and packs
+        them in a dictionary.
 
         For instance, if a batch has three graphs g1, g2 and g3 with node
         features (x1, x2, x3) and adjacency matrices (a1, a2, a3), this method
-        will return:
+        will return a dictionary:
 
-        ```
-        a_list = [a1, a2, a3]
-        x_list = [x1, x2, x3]
+        ```python
+        >>> {'a_list': [a1, a2, a3], 'x_list': [x1, x2, x3]}
         ```
 
-        If `return_dict=True`, the lists are wrapped in a dictionary:
-
-        ```
-        {'a_list': [a1, a2, a3],
-         'x_list': [x1, x2, x3]}
-        ```
-
-         this is useful for passing the packed batch to `data.utils.to_batch()`
-         and `data.utils.to_disjoint()` without knowing a-priori what are the
-         attributes of the graphs.
-
-        :param batch: a list of Graphs
-        :param return_dict: whether to return the lists as element of a dictionary.
-        :return: the batch packed into lists, by attribute type.
+        :param batch: a list of `Graph` objects.
         """
         output = [list(elem) for elem in zip(*[g.numpy() for g in batch])]
-        if return_dict:
-            keys = [k + "_list" for k in self.dataset.signature.keys()]
-            return {k: v for k, v in zip(keys, output)}
-        else:
-            return output
+        keys = [k + "_list" for k in self.dataset.signature.keys()]
+        return dict(zip(keys, output))
 
     @property
     def steps_per_epoch(self):
+        """
+        :return: the number of batches of size `self.batch_size` in the dataset (i.e.,
+        how many batches are in an epoch).
+        """
         return int(np.ceil(len(self.dataset) / self.batch_size))
 
 
 class SingleLoader(Loader):
     """
-    A Loader for
-    [single mode](https://graphneural.network/data-modes/#single-mode).
+    A Loader for [single mode](https://graphneural.network/data-modes/#single-mode).
 
     This loader produces Tensors representing a single graph. As such, it can
     only be used with Datasets of length 1 and the `batch_size` cannot be set.
@@ -173,12 +178,11 @@ class SingleLoader(Loader):
 
     **Arguments**
 
-    - `dataset`: a graph Dataset;
-    - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
+    - `dataset`: a `spektral.data.Dataset` object with only one graph;
+    - `epochs`: int, number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
-    - `shuffle`: whether to shuffle the data at the start of each epoch;
-    - `sample_weights`: if given, these will be appended to the output
-    automatically.
+    - `shuffle`: bool, whether to shuffle the data at the start of each epoch;
+    - `sample_weights`: Numpy array, will be appended to the output automatically.
 
     **Output**
 
@@ -192,10 +196,9 @@ class SingleLoader(Loader):
     SparseTensors);
     - `e`: same as `dataset[0].e`;
 
-    `labels` is the same as `datsaset[0].y`.
-    `sample_weights` is the same object passed to the constructor.
-
+    `labels` is the same as `dataset[0].y`.
 
+    `sample_weights` is the same array passed when creating the loader.
     """
 
     def __init__(self, dataset, epochs=None, sample_weights=None):
@@ -208,20 +211,29 @@ class SingleLoader(Loader):
         super().__init__(dataset, batch_size=1, epochs=epochs, shuffle=False)
 
     def collate(self, batch):
-        graph = batch[0]
-        output = graph.numpy()
+        packed = self.pack(batch)
 
-        # Sparse matrices to SparseTensors
-        output = list(output)
-        for i in range(len(output)):
-            if sp.issparse(output[i]):
-                output[i] = sp_matrix_to_sp_tensor(output[i])
-        output = tuple(output)
+        y = packed.pop("y_list", None)
+        if y is not None:
+            y = collate_labels_disjoint(y, node_level=True)
 
-        output = (output[:-1], output[-1])
+        output = to_disjoint(**packed)
+        output = output[:-1]  # Discard batch index
+        output = sp_matrices_to_sp_tensors(output)
+
+        if len(output) == 1:
+            output = output[0]
+
+        output = (output,)
+        if y is not None:
+            output += (y,)
         if self.sample_weights is not None:
             output += (self.sample_weights,)
-        return tuple(output)
+
+        if len(output) == 1:
+            output = output[0]  # Again, in case there are no targets and no SW
+
+        return output
 
     def load(self):
         output = self.collate(self.dataset)
@@ -230,8 +242,7 @@ class SingleLoader(Loader):
 
 class DisjointLoader(Loader):
     """
-    A Loader for
-    [disjoint mode](https://graphneural.network/data-modes/#disjoint-mode).
+    A Loader for [disjoint mode](https://graphneural.network/data-modes/#disjoint-mode).
 
     This loader represents a batch of graphs via their disjoint union.
 
@@ -250,6 +261,8 @@ class DisjointLoader(Loader):
     **Arguments**
 
     - `dataset`: a graph Dataset;
+    - `node_level`: bool, if `True` stack the labels vertically for node-level
+    prediction;
     - `batch_size`: size of the mini-batches;
     - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
@@ -263,7 +276,8 @@ class DisjointLoader(Loader):
 
     - `x`: node attributes of shape `[n_nodes, n_node_features]`;
     - `a`: adjacency matrices of shape `[n_nodes, n_nodes]`;
-    - `e`: edge attributes of shape `[n_edges, n_edge_features]`.
+    - `e`: edge attributes of shape `[n_edges, n_edge_features]`;
+    - `i`: batch index of shape `[n_nodes]`.
 
     `labels` have shape `[batch, n_labels]` if `node_level=False` or
     `[n_nodes, n_labels]` otherwise.
@@ -274,23 +288,20 @@ class DisjointLoader(Loader):
         self, dataset, node_level=False, batch_size=1, epochs=None, shuffle=True
     ):
         self.node_level = node_level
-        super(DisjointLoader, self).__init__(
-            dataset, batch_size=batch_size, epochs=epochs, shuffle=shuffle
-        )
+        super().__init__(dataset, batch_size=batch_size, epochs=epochs, shuffle=shuffle)
 
     def collate(self, batch):
-        packed = self.pack(batch, return_dict=True)
-        y = None
-        if "y" in self.dataset.signature:
-            y = packed.pop("y_list")
-            y = np.vstack(y) if self.node_level else np.array(y)
+        packed = self.pack(batch)
+
+        y = packed.pop("y_list", None)
+        if y is not None:
+            y = collate_labels_disjoint(y, node_level=self.node_level)
 
         output = to_disjoint(**packed)
-        output = list(output)
-        for i in range(len(output)):
-            if sp.issparse(output[i]):
-                output[i] = sp_matrix_to_sp_tensor(output[i])
-        output = tuple(output)
+        output = sp_matrices_to_sp_tensors(output)
+
+        if len(output) == 1:
+            output = output[0]
 
         if y is None:
             return output
@@ -311,12 +322,11 @@ class DisjointLoader(Loader):
         Adjacency matrix has shape [n_nodes, n_nodes]
         Node features have shape [n_nodes, n_node_features]
         Edge features have shape [n_edges, n_edge_features]
-        Targets have shape [..., n_labels]
+        Targets have shape [*, n_labels]
         """
         signature = self.dataset.signature
         if "y" in signature:
-            if not self.node_level:
-                signature["y"]["shape"] = prepend_none(signature["y"]["shape"])
+            signature["y"]["shape"] = prepend_none(signature["y"]["shape"])
         if "a" in signature:
             signature["a"]["spec"] = tf.SparseTensorSpec
 
@@ -330,8 +340,7 @@ class DisjointLoader(Loader):
 
 class BatchLoader(Loader):
     """
-    A Loader for
-    [batch mode](https://graphneural.network/data-modes/#batch-mode).
+    A Loader for [batch mode](https://graphneural.network/data-modes/#batch-mode).
 
     This loader returns batches of graphs stacked along an extra dimension,
     with all "node" dimensions padded to be equal among all graphs.
@@ -339,29 +348,40 @@ class BatchLoader(Loader):
     If `n_max` is the number of nodes of the biggest graph in the batch, then
     the padding consist of adding zeros to the node features, adjacency matrix,
     and edge attributes of each graph so that they have shapes
-    `(n_max, n_node_features)`, `(n_max, n_max)`, and
-    `(n_max, n_max, n_edge_features)` respectively.
+    `[n_max, n_node_features]`, `[n_max, n_max]`, and
+    `[n_max, n_max, n_edge_features]` respectively.
 
     The zero-padding is done batch-wise, which saves up memory at the cost of
     more computation. If latency is an issue but memory isn't, or if the
     dataset has graphs with a similar number of nodes, you can use
-    the `PackedBatchLoader` that first zero-pads all the dataset and then
+    the `PackedBatchLoader` that zero-pads all the dataset once and then
     iterates over it.
 
     Note that the adjacency matrix and edge attributes are returned as dense
-    arrays (mostly due to the lack of support for sparse tensor operations for
-    rank >2).
+    arrays.
 
-    Only graph-level labels are supported with this loader (i.e., labels are not
-    zero-padded because they are assumed to have no "node" dimensions).
+    if `mask=True`, node attributes will be extended with a binary mask that indicates
+    valid nodes (the last feature of each node will be 1 if the node was originally in
+    the graph and 0 if it is a fake node added by zero-padding).
+
+    Use this flag in conjunction with layers.base.GraphMasking to start the propagation
+    of masks in a model (necessary for node-level prediction and models that use a
+    dense pooling layer like DiffPool or MinCutPool).
+
+    If `node_level=False`, the labels are interpreted as graph-level labels and
+    are returned as an array of shape `[batch, n_labels]`.
+    If `node_level=True`, then the labels are padded along the node dimension and are
+    returned as an array of shape `[batch, n_max, n_labels]`.
 
     **Arguments**
 
     - `dataset`: a graph Dataset;
-    - `batch_size`: size of the mini-batches;
-    - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
+    - `mask`: bool, whether to add a mask to the node features;
+    - `batch_size`: int, size of the mini-batches;
+    - `epochs`: int, number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
-    - `shuffle`: whether to shuffle the data at the start of each epoch.
+    - `shuffle`: bool, whether to shuffle the data at the start of each epoch;
+    - `node_level`: bool, if `True` pad the labels along the node dimension;
 
     **Output**
 
@@ -371,16 +391,36 @@ class BatchLoader(Loader):
 
     - `x`: node attributes of shape `[batch, n_max, n_node_features]`;
     - `a`: adjacency matrices of shape `[batch, n_max, n_max]`;
-    - `e`: edge attributes of shape `[batch, n_edges, n_edge_features]`.
+    - `e`: edge attributes of shape `[batch, n_max, n_max, n_edge_features]`.
 
-    `labels` have shape `[batch, n_labels]`.
+    `labels` have shape `[batch, n_labels]` if `node_level=False` or
+    `[batch, n_max, n_labels]` otherwise.
     """
 
+    def __init__(
+        self,
+        dataset,
+        mask=False,
+        batch_size=1,
+        epochs=None,
+        shuffle=True,
+        node_level=False,
+    ):
+        self.mask = mask
+        self.node_level = node_level
+        self.signature = dataset.signature
+        super().__init__(dataset, batch_size=batch_size, epochs=epochs, shuffle=shuffle)
+
     def collate(self, batch):
-        packed = self.pack(batch, return_dict=True)
-        y = np.array(packed.pop("y_list")) if "y" in self.dataset.signature else None
+        packed = self.pack(batch)
+
+        y = packed.pop("y_list", None)
+        if y is not None:
+            y = collate_labels_batch(y, node_level=self.node_level)
+
+        output = to_batch(**packed, mask=self.mask)
+        output = sp_matrices_to_sp_tensors(output)
 
-        output = to_batch(**packed)
         if len(output) == 1:
             output = output[0]
 
@@ -394,17 +434,25 @@ class BatchLoader(Loader):
         Adjacency matrix has shape [batch, n_nodes, n_nodes]
         Node features have shape [batch, n_nodes, n_node_features]
         Edge features have shape [batch, n_nodes, n_nodes, n_edge_features]
-        Targets have shape [batch, ..., n_labels]
+        Labels have shape [batch, n_labels]
         """
-        signature = self.dataset.signature
+        signature = self.signature
         for k in signature:
             signature[k]["shape"] = prepend_none(signature[k]["shape"])
+        if "x" in signature and self.mask:
+            # In case we have a mask, the mask is concatenated to the features
+            signature["x"]["shape"] = signature["x"]["shape"][:-1] + (
+                signature["x"]["shape"][-1] + 1,
+            )
         if "a" in signature:
             # Adjacency matrix in batch mode is dense
             signature["a"]["spec"] = tf.TensorSpec
         if "e" in signature:
             # Edge attributes have an extra None dimension in batch mode
             signature["e"]["shape"] = prepend_none(signature["e"]["shape"])
+        if "y" in signature and self.node_level:
+            # Node labels have an extra None dimension
+            signature["y"]["shape"] = prepend_none(signature["y"]["shape"])
 
         return to_tf_signature(signature)
 
@@ -429,10 +477,12 @@ class PackedBatchLoader(BatchLoader):
     **Arguments**
 
     - `dataset`: a graph Dataset;
-    - `batch_size`: size of the mini-batches;
-    - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
+    - `mask`: bool, whether to add a mask to the node features;
+    - `batch_size`: int, size of the mini-batches;
+    - `epochs`: int, number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
-    - `shuffle`: whether to shuffle the data at the start of each epoch.
+    - `shuffle`: bool, whether to shuffle the data at the start of each epoch;
+    - `node_level`: bool, if `True` pad the labels along the node dimension;
 
     **Output**
 
@@ -442,18 +492,38 @@ class PackedBatchLoader(BatchLoader):
 
     - `x`: node attributes of shape `[batch, n_max, n_node_features]`;
     - `a`: adjacency matrices of shape `[batch, n_max, n_max]`;
-    - `e`: edge attributes of shape `[batch, n_max, n_edge_features]`.
+    - `e`: edge attributes of shape `[batch, n_max, n_max, n_edge_features]`.
 
-    `labels` have shape `[batch, ..., n_labels]`.
+    `labels` have shape `[batch, n_labels]` if `node_level=False` or
+    `[batch, n_max, n_labels]` otherwise.
     """
 
-    def __init__(self, dataset, batch_size=1, epochs=None, shuffle=True):
-        super().__init__(dataset, batch_size=batch_size, epochs=epochs, shuffle=shuffle)
+    def __init__(
+        self,
+        dataset,
+        mask=False,
+        batch_size=1,
+        epochs=None,
+        shuffle=True,
+        node_level=False,
+    ):
+        super().__init__(
+            dataset,
+            mask=mask,
+            batch_size=batch_size,
+            epochs=epochs,
+            shuffle=shuffle,
+            node_level=node_level,
+        )
 
         # Drop the Dataset container and work on packed tensors directly
-        packed = self.pack(self.dataset, return_dict=True)
-        y = np.array(packed.pop("y_list")) if "y" in dataset.signature else None
-        self.dataset = to_batch(**packed)
+        packed = self.pack(self.dataset)
+
+        y = packed.pop("y_list", None)
+        if y is not None:
+            y = collate_labels_batch(y, node_level=self.node_level)
+
+        self.dataset = to_batch(**packed, mask=mask)
         if y is not None:
             self.dataset += (y,)
 
@@ -477,8 +547,7 @@ class PackedBatchLoader(BatchLoader):
 
 class MixedLoader(Loader):
     """
-    A Loader for
-    [mixed mode](https://graphneural.network/data-modes/#mixed-mode).
+    A Loader for [mixed mode](https://graphneural.network/data-modes/#mixed-mode).
 
     This loader returns batches where the node and edge attributes are stacked
     along an extra dimension, but the adjacency matrix is shared by all graphs.
@@ -490,10 +559,10 @@ class MixedLoader(Loader):
     **Arguments**
 
     - `dataset`: a graph Dataset;
-    - `batch_size`: size of the mini-batches;
-    - `epochs`: number of epochs to iterate over the dataset. By default (`None`)
+    - `batch_size`: int, size of the mini-batches;
+    - `epochs`: int, number of epochs to iterate over the dataset. By default (`None`)
     iterates indefinitely;
-    - `shuffle`: whether to shuffle the data at the start of each epoch.
+    - `shuffle`: bool, whether to shuffle the data at the start of each epoch.
 
     **Output**
 
@@ -526,11 +595,16 @@ class MixedLoader(Loader):
         super().__init__(dataset, batch_size=batch_size, epochs=epochs, shuffle=shuffle)
 
     def collate(self, batch):
-        packed = self.pack(batch, return_dict=True)
-        y = np.array(packed.pop("y_list")) if "y" in self.dataset.signature else None
-
+        packed = self.pack(batch)
         packed["a"] = self.dataset.a
+
+        y = packed.pop("y_list", None)
+        if y is not None:
+            y = np.array(y)
+
         output = to_mixed(**packed)
+        output = sp_matrices_to_sp_tensors(output)
+
         if len(output) == 1:
             output = output[0]
 
@@ -551,4 +625,9 @@ class MixedLoader(Loader):
             if k in signature:
                 signature[k]["shape"] = prepend_none(signature[k]["shape"])
 
+        signature["a"] = dict()
+        signature["a"]["spec"] = get_spec(self.dataset.a)
+        signature["a"]["shape"] = (None, None)
+        signature["a"]["dtype"] = tf.as_dtype(self.dataset.a.dtype)
+
         return to_tf_signature(signature)
