@@ -3,6 +3,7 @@ import tensorflow as tf
 from scipy import sparse as sp
 
 from spektral.utils import pad_jagged_array
+from spektral.utils.sparse import sp_matrix_to_sp_tensor
 
 
 def to_disjoint(x_list=None, a_list=None, e_list=None):
@@ -64,7 +65,7 @@ def to_disjoint(x_list=None, a_list=None, e_list=None):
     return tuple(out for out in [x_out, a_out, e_out, i_out] if out is not None)
 
 
-def to_batch(x_list=None, a_list=None, e_list=None):
+def to_batch(x_list=None, a_list=None, e_list=None, mask=False):
     """
     Converts lists of node features, adjacency matrices and edge features to
     [batch mode](https://graphneural.network/data-modes/#batch-mode),
@@ -90,6 +91,11 @@ def to_batch(x_list=None, a_list=None, e_list=None):
     `(n_nodes, n_nodes)`;
     :param e_list: a list of np.arrays of shape
     `(n_nodes, n_nodes, n_edge_features)` or `(n_edges, n_edge_features)`;
+    :param mask: bool, if True, node attributes will be extended with a binary mask that
+    indicates valid nodes (the last feature of each node will be 1 if the node is valid
+    and 0 otherwise). Use this flag in conjunction with layers.base.GraphMasking to
+    start the propagation of masks in a model.
+
     :return: only if the corresponding list is given as input:
 
         -  `x`: np.array of shape `(batch, n_max, n_node_features)`;
@@ -104,6 +110,8 @@ def to_batch(x_list=None, a_list=None, e_list=None):
     # Node features
     x_out = None
     if x_list is not None:
+        if mask:
+            x_list = [np.concatenate((x, np.ones((x.shape[0], 1))), -1) for x in x_list]
         x_out = pad_jagged_array(x_list, (n_max, -1))
 
     # Adjacency matrix
@@ -184,7 +192,7 @@ def batch_generator(data, batch_size=32, epochs=None, shuffle=True):
         data = [data]
     if len(data) < 1:
         raise ValueError("data cannot be empty")
-    if len(set([len(item) for item in data])) > 1:
+    if len({len(item) for item in data}) > 1:
         raise ValueError("All inputs must have the same __len__")
 
     if epochs is None or epochs == -1:
@@ -246,3 +254,44 @@ def to_tf_signature(signature):
         output = (output, spec(shape, dtype))
 
     return output
+
+
+def sp_matrices_to_sp_tensors(inputs):
+    inputs = list(inputs)
+    for i in range(len(inputs)):
+        if sp.issparse(inputs[i]):
+            inputs[i] = sp_matrix_to_sp_tensor(inputs[i])
+    return tuple(inputs)
+
+
+def collate_labels_disjoint(y_list, node_level=False):
+    """
+    Collates the given list of labels for disjoint mode.
+
+    If `node_level=False`, the labels can be scalars or must have shape `[n_labels, ]`.
+    If `node_level=True`, the labels must have shape `[n_nodes, ]` (i.e., a scalar label
+    for each node) or `[n_nodes, n_labels]`.
+
+    :param y_list: a list of np.arrays or scalars.
+    :param node_level: bool, whether to pack the labels as node-level or graph-level.
+    :return:
+        - If `node_level=False`: a np.array of shape `[len(y_list), n_labels]`.
+        - If `node_level=True`: a np.array of shape `[n_nodes_total, n_labels]`, where
+        `n_nodes_total = sum(y.shape[0] for y in y_list)`.
+    """
+    if node_level:
+        if len(np.shape(y_list[0])) == 1:
+            y_list = [y_[:, None] for y_ in y_list]
+        return np.vstack(y_list)
+    else:
+        if len(np.shape(y_list[0])) == 0:
+            y_list = [np.array([y_]) for y_ in y_list]
+        return np.array(y_list)
+
+
+def collate_labels_batch(y_list, node_level=False):
+    if node_level:
+        n_max = max([x.shape[0] for x in y_list])
+        return pad_jagged_array(y_list, (n_max, -1))
+    else:
+        return np.array(y_list)
