@@ -1,3 +1,4 @@
+import os
 import os.path as osp
 import shutil
 from glob import glob
@@ -40,35 +41,39 @@ class ModelNet(Dataset):
     }
 
     def __init__(self, name, test=False, n_jobs=-1, **kwargs):
-        if name not in self.available_datasets:
+        if name not in self.available_datasets():
             raise ValueError(
-                "Unknown dataset {}. Possible: {}".format(name, self.available_datasets)
+                "Unknown dataset {}. See {}.available_datasets() for a complete list of"
+                "available datasets.".format(name, self.__class__.__name__)
             )
         self.name = name
         self.test = test
         self.n_jobs = n_jobs
-        self.true_path = osp.join(self.path, "ModelNet" + self.name)
         super().__init__(**kwargs)
 
+    @property
+    def path(self):
+        return osp.join(super().path, self.name)
+
     def read(self):
-        folders = glob(osp.join(self.true_path, "*", ""))
+        folders = glob(osp.join(self.path, "*", ""))
         dataset = "test" if self.test else "train"
-        classes = [f.split("/")[-2] for f in folders]
+        classes = [f.split(os.path.sep)[-2] for f in folders]
         n_out = len(classes)
 
         print("Loading data")
 
-        def load(fname):
+        def load(fname, class_i):
             graph = load_off(fname)
-            graph.y = one_hot(i, n_out)
+            graph.y = one_hot(class_i, n_out)
             return graph
 
         output = []
         for i, c in enumerate(tqdm(classes)):
-            fnames = osp.join(self.true_path, c, dataset, "{}_*.off".format(c))
+            fnames = osp.join(self.path, c, dataset, "{}_*.off".format(c))
             fnames = glob(fnames)
             output_partial = Parallel(n_jobs=self.n_jobs)(
-                delayed(load)(fname) for fname in fnames
+                delayed(load)(fname, i) for fname in fnames
             )
             output.extend(output_partial)
 
@@ -78,8 +83,15 @@ class ModelNet(Dataset):
         print("Downloading ModelNet{} dataset.".format(self.name))
         url = self.url[self.name]
         download_file(url, self.path, "ModelNet" + self.name + ".zip")
+
+        # Datasets are zipped in a folder: unpack them
+        parent = self.path
+        subfolder = osp.join(self.path, "ModelNet" + self.name)
+        for filename in os.listdir(subfolder):
+            shutil.move(osp.join(subfolder, filename), osp.join(parent, filename))
+        os.rmdir(subfolder)
         shutil.rmtree(osp.join(self.path, "__MACOSX"), ignore_errors=True)
 
-    @property
-    def available_datasets(self):
+    @staticmethod
+    def available_datasets():
         return ["10", "40"]
