@@ -1,27 +1,22 @@
 import glob
 import os
 import shutil
-import zipfile
 from os import path as osp
 from urllib.error import URLError
 
 import numpy as np
 import pandas as pd
-import requests
-import scipy.sparse as sp
 from sklearn.preprocessing import OneHotEncoder, StandardScaler
 
 from spektral.data import Dataset, Graph
 from spektral.datasets.utils import download_file
-from spektral.utils import io
-
-DATA_PATH = osp.expanduser("~/.spektral/datasets/")
+from spektral.utils import io, sparse
 
 
 class TUDataset(Dataset):
     """
     The Benchmark Data Sets for Graph Kernels from TU Dortmund
-    ([link](https://ls11-www.cs.tu-dortmund.de/staff/morris/graphkerneldatasets)).
+    ([link](https://chrsmrrs.github.io/datasets/docs/datasets/)).
 
     Node features are computed by concatenating the following features for
     each node:
@@ -30,8 +25,10 @@ class TUDataset(Dataset):
     - node labels, if available, one-hot encoded.
 
     Some datasets might not have node features at all. In this case, attempting
-    to use the dataset with a Loader will result in a crash. In this case,
-    you should set the features manually by iterating over the `graph` list.
+    to use the dataset with a Loader will result in a crash. You can create
+    node features using some of the transforms available in `spektral.transforms`
+    or you can define your own features by accessing the individual samples in
+    the `graph` attribute of the dataset (which is a list of `Graph` objects).
 
     Edge features are computed by concatenating the following features for
     each node:
@@ -39,8 +36,12 @@ class TUDataset(Dataset):
     - edge attributes, if available;
     - edge labels, if available, one-hot encoded.
 
-    Graph labels are provided for each dataset. See the dataset's README in
-    ~/.spektral/datasets/TUD/`name`/ for details about each dataset.
+    Graph labels are provided for each dataset.
+
+    Specific details about each individual dataset can be found in
+    `~/.spektral/datasets/TUDataset/<dataset name>/README.md`, after the dataset
+    has been downloaded locally (datasets are downloaded automatically upon
+    calling `TUDataset('<dataset name>')` the first time).
 
     **Arguments**
 
@@ -55,15 +56,18 @@ class TUDataset(Dataset):
     )
 
     def __init__(self, name, clean=False, **kwargs):
+        if name not in self.available_datasets():
+            raise ValueError(
+                "Unknown dataset {}. See {}.available_datasets() for a complete list of"
+                "available datasets.".format(name, self.__class__.__name__)
+            )
         self.name = name
         self.clean = clean
         super().__init__(**kwargs)
 
     @property
     def path(self):
-        return osp.join(
-            super(TUDataset, self).path, self.name + ("_clean" if self.clean else "")
-        )
+        return osp.join(super().path, self.name + ("_clean" if self.clean else ""))
 
     def download(self):
         print(
@@ -95,7 +99,7 @@ class TUDataset(Dataset):
         n_nodes = np.bincount(node_batch_index)
         n_nodes_cum = np.concatenate(([0], np.cumsum(n_nodes)[:-1]))
 
-        # Adjacency matrix
+        # Read edge lists
         edges = io.load_txt(fname_template.format("A"), delimiter=",").astype(int) - 1
         # Remove duplicates and self-loops from edges
         _, mask = np.unique(edges, axis=0, return_index=True)
@@ -105,15 +109,7 @@ class TUDataset(Dataset):
         edge_batch_idx = node_batch_index[edges[:, 0]]
         n_edges = np.bincount(edge_batch_idx)
         n_edges_cum = np.cumsum(n_edges[:-1])
-        edge_lists = np.split(edges - n_nodes_cum[edge_batch_idx, None], n_edges_cum)
-        # Create sparse adjacency matrices
-        a_list = [
-            sp.csr_matrix(
-                (np.ones_like(el[:, 0]), (el[:, 0], el[:, 1])),
-                shape=(n_nodes[i], n_nodes[i]),
-            )
-            for i, el in enumerate(edge_lists)
-        ]
+        el_list = np.split(edges - n_nodes_cum[edge_batch_idx, None], n_edges_cum)
 
         # Node features
         x_list = []
@@ -161,11 +157,29 @@ class TUDataset(Dataset):
             )
             e_list.append(e_labs)
         if len(e_list) > 0:
+            e_available = True
             e_list = np.concatenate(e_list, -1)
             e_list = np.split(e_list, n_edges_cum)
         else:
+            e_available = False
             e_list = [None] * len(n_nodes)
 
+        # Create sparse adjacency matrices and re-sort edge attributes in lexicographic
+        # order
+        a_e_list = [
+            sparse.edge_index_to_matrix(
+                edge_index=el,
+                edge_weight=np.ones(el.shape[0]),
+                edge_features=e,
+                shape=(n, n),
+            )
+            for el, e, n in zip(el_list, e_list, n_nodes)
+        ]
+        if e_available:
+            a_list, e_list = list(zip(*a_e_list))
+        else:
+            a_list = a_e_list
+
         # Labels
         if "graph_attributes" in available:
             labels = io.load_txt(fname_template.format("graph_attributes"))
@@ -182,8 +196,8 @@ class TUDataset(Dataset):
             for x, a, e, y in zip(x_list, a_list, e_list, labels)
         ]
 
-    @property
-    def available_datasets(self):
+    @staticmethod
+    def available_datasets():
         url = "https://chrsmrrs.github.io/datasets/docs/datasets/"
         try:
             tables = pd.read_html(url)
