@@ -5,7 +5,15 @@ import zipfile
 import requests
 from tqdm import tqdm
 
-DATASET_FOLDER = osp.expanduser("~/.spektral/datasets")
+_dataset_folder = os.path.join("~", "spektral", "datasets")
+_config_path = osp.expanduser(os.path.join("~", "spektral", "config.json"))
+if osp.isfile(_config_path):
+    import json
+
+    with open(_config_path) as fh:
+        _config = json.load(fh)
+        _dataset_folder = _config.get("dataset_folder", _dataset_folder)
+DATASET_FOLDER = osp.expanduser(_dataset_folder)
 
 
 def download_file(url, datadir, fname, progress=True, extract=True):
