@@ -103,9 +103,10 @@ class APPNPConv(Conv):
             bias_regularizer=self.bias_regularizer,
             kernel_constraint=self.kernel_constraint,
             bias_constraint=self.bias_constraint,
+            dtype=self.dtype,
         )
         mlp_layers = []
-        for i, channels in enumerate(self.mlp_hidden):
+        for channels in self.mlp_hidden:
             mlp_layers.extend(
                 [
                     Dropout(self.dropout_rate),
@@ -116,14 +117,17 @@ class APPNPConv(Conv):
         self.mlp = Sequential(mlp_layers)
         self.built = True
 
-    def call(self, inputs):
+    def call(self, inputs, mask=None):
         x, a = inputs
 
         mlp_out = self.mlp(x)
-        z = mlp_out
-        for k in range(self.propagations):
-            z = (1 - self.alpha) * ops.modal_dot(a, z) + self.alpha * mlp_out
-        output = self.activation(z)
+        output = mlp_out
+        for _ in range(self.propagations):
+            output = (1 - self.alpha) * ops.modal_dot(a, output) + self.alpha * mlp_out
+
+        if mask is not None:
+            output *= mask[0]
+        output = self.activation(output)
 
         return output
 
