@@ -59,6 +59,7 @@ class GATConv(Conv):
     - `dropout_rate`: internal dropout rate for attention coefficients;
     - `return_attn_coef`: if True, return the attention coefficients for
     the given input (one n_nodes x n_nodes matrix for each head).
+    - `add_self_loops`: if True, add self loops to the adjacency matrix.
     - `activation`: activation function;
     - `use_bias`: bool, add a bias vector to the output;
     - `kernel_initializer`: initializer for the weights;
@@ -81,6 +82,7 @@ class GATConv(Conv):
         concat_heads=True,
         dropout_rate=0.5,
         return_attn_coef=False,
+        add_self_loops=True,
         activation=None,
         use_bias=True,
         kernel_initializer="glorot_uniform",
@@ -112,6 +114,7 @@ class GATConv(Conv):
         self.concat_heads = concat_heads
         self.dropout_rate = dropout_rate
         self.return_attn_coef = return_attn_coef
+        self.add_self_loops = add_self_loops
         self.attn_kernel_initializer = initializers.get(attn_kernel_initializer)
         self.attn_kernel_regularizer = regularizers.get(attn_kernel_regularizer)
         self.attn_kernel_constraint = constraints.get(attn_kernel_constraint)
@@ -155,10 +158,10 @@ class GATConv(Conv):
                 name="bias",
             )
 
-        self.dropout = Dropout(self.dropout_rate)
+        self.dropout = Dropout(self.dropout_rate, dtype=self.dtype)
         self.built = True
 
-    def call(self, inputs):
+    def call(self, inputs, mask=None):
         x, a = inputs
 
         mode = ops.autodetect_mode(x, a)
@@ -170,15 +173,17 @@ class GATConv(Conv):
             output, attn_coef = self._call_dense(x, a)
 
         if self.concat_heads:
-            shape = output.shape[:-2] + [self.attn_heads * self.channels]
-            shape = [d if d is not None else -1 for d in shape]
+            shape = tf.concat(
+                (tf.shape(output)[:-2], [self.attn_heads * self.channels]), axis=0
+            )
             output = tf.reshape(output, shape)
         else:
             output = tf.reduce_mean(output, axis=-2)
 
         if self.use_bias:
             output += self.bias
-
+        if mask is not None:
+            output *= mask[0]
         output = self.activation(output)
 
         if self.return_attn_coef:
@@ -195,7 +200,8 @@ class GATConv(Conv):
         # Prepare message-passing
         indices = a.indices
         N = tf.shape(x, out_type=indices.dtype)[-2]
-        indices = ops.add_self_loops_indices(indices, N)
+        if self.add_self_loops:
+            indices = ops.add_self_loops_indices(indices, N)
         targets, sources = indices[:, 1], indices[:, 0]
 
         # Update node features
@@ -222,8 +228,8 @@ class GATConv(Conv):
 
     def _call_dense(self, x, a):
         shape = tf.shape(a)[:-1]
-        a = tf.linalg.set_diag(a, tf.zeros(shape, a.dtype))
-        a = tf.linalg.set_diag(a, tf.ones(shape, a.dtype))
+        if self.add_self_loops:
+            a = tf.linalg.set_diag(a, tf.ones(shape, a.dtype))
         x = tf.einsum("...NI , IHO -> ...NHO", x, self.kernel)
         attn_for_self = tf.einsum("...NHI , IHO -> ...NHO", x, self.attn_kernel_self)
         attn_for_neighs = tf.einsum(
@@ -234,7 +240,8 @@ class GATConv(Conv):
         attn_coef = attn_for_self + attn_for_neighs
         attn_coef = tf.nn.leaky_relu(attn_coef, alpha=0.2)
 
-        mask = -10e9 * (1.0 - a)
+        mask = tf.where(a == 0.0, -10e9, 0.0)
+        mask = tf.cast(mask, dtype=attn_coef.dtype)
         attn_coef += mask[..., None, :]
         attn_coef = tf.nn.softmax(attn_coef, axis=-1)
         attn_coef_drop = self.dropout(attn_coef)
