@@ -97,11 +97,9 @@ class GCSConv(Conv):
                 regularizer=self.bias_regularizer,
                 constraint=self.bias_constraint,
             )
-        else:
-            self.bias = None
         self.built = True
 
-    def call(self, inputs):
+    def call(self, inputs, mask=None):
         x, a = inputs
 
         output = K.dot(x, self.kernel_1)
@@ -111,8 +109,10 @@ class GCSConv(Conv):
 
         if self.use_bias:
             output = K.bias_add(output, self.bias)
-        if self.activation is not None:
-            output = self.activation(output)
+        if mask is not None:
+            output *= mask[0]
+        output = self.activation(output)
+
         return output
 
     @property
