@@ -27,6 +27,8 @@ class GeneralGNN(Model):
     > [Design Space for Graph Neural Networks](https://arxiv.org/abs/2011.08843)<br>
     > Jiaxuan You, Rex Ying, Jure Leskovec
 
+    **Mode**: single, disjoint, mixed.
+
     The default parameters of the model are selected according to the best
     results obtained in the paper, and should provide a good performance on
     many node-level and graph-level tasks, without modifications.
@@ -101,6 +103,20 @@ class GeneralGNN(Model):
         pool="sum",
     ):
         super().__init__()
+        self.config = {
+            "output": output,
+            "activation": activation,
+            "hidden": hidden,
+            "message_passing": message_passing,
+            "pre_process": pre_process,
+            "post_process": post_process,
+            "connectivity": connectivity,
+            "batch_norm": batch_norm,
+            "dropout": dropout,
+            "aggregate": aggregate,
+            "hidden_activation": hidden_activation,
+            "pool": pool,
+        }
 
         # Connectivity function
         if connectivity is None:
@@ -160,12 +176,15 @@ class GeneralGNN(Model):
                 out = z
         # Global pooling
         if self.pool is not None:
-            out = self.pool([out] + [i] if i is not None else [])
+            out = self.pool([out] + ([i] if i is not None else []))
         # Post-process
         out = self.post(out)
 
         return out
 
+    def get_config(self):
+        return self.config
+
 
 class MLP(Model):
     def __init__(
@@ -179,6 +198,15 @@ class MLP(Model):
         final_activation=None,
     ):
         super().__init__()
+        self.config = {
+            "output": output,
+            "hidden": hidden,
+            "layers": layers,
+            "batch_norm": batch_norm,
+            "dropout": dropout,
+            "activation": activation,
+            "final_activation": final_activation,
+        }
         self.batch_norm = batch_norm
         self.dropout_rate = dropout
 
@@ -196,3 +224,6 @@ class MLP(Model):
 
     def call(self, inputs):
         return self.mlp(inputs)
+
+    def get_config(self):
+        return self.config
