@@ -1,7 +1,7 @@
-from spektral.layers.ops import sp_matrix_to_sp_tensor
+from spektral.utils.sparse import sp_matrix_to_sp_tensor
 
 
-class AdjToSpTensor(object):
+class AdjToSpTensor:
     """
     Converts the adjacency matrix to a SparseTensor.
     """
